////////////////////////////////////////////////////////////////
//
// Copyright (C) 2005 Affymetrix, Inc.
//
// This program is free software; you can redistribute it and/or modify 
// it under the terms of the GNU General Public License (version 2) as 
// published by the Free Software Foundation.
// 
// This program is distributed in the hope that it will be useful, 
// but WITHOUT ANY WARRANTY; without even the implied warranty of 
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
// General Public License for more details.
// 
// You should have received a copy of the GNU General Public License 
// along with this program;if not, write to the 
// 
// Free Software Foundation, Inc., 
// 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
//
////////////////////////////////////////////////////////////////

/**
 * @file   PgOptionsTest.cpp
 * @author Chuck Sugnet
 * @date   Tue May  3 13:14:45 2005
 * 
 * @brief  Testing program for the PgOptions module.
 * 
 */
#ifndef PGOPTIONSTEST_H
#define PGOPTIONSTEST_H

#include "PgOptions.h"
#include "Convert.h"
#include <cppunit/extensions/HelperMacros.h>
#include <cppunit/CompilerOutputter.h>
#include <cppunit/extensions/TestFactoryRegistry.h>
#include <cppunit/ui/text/TestRunner.h>


static PgOptions::PgOpt INTEGER_OPT = {
  "i", "integer-opt", PgOptions::INT_OPT,
  "Tests to see what an integer value looks like. The description of this option should wrap.",
  "10", NULL
};

static PgOptions::PgOpt FLOAT_OPT = {
  "f", "float-opt", PgOptions::FLOAT_OPT,
  "Tests to see what an float value looks like.",
  "10.5", NULL
};

static PgOptions::PgOpt STRING_OPT = {
  "s", "string-opt", PgOptions::STRING_OPT,
  "Tests to see what an string value looks like.",
  "stringOption", NULL
};

static PgOptions::PgOpt STRING_OPT2 = {
  "s2", "string-opt2-really-long", PgOptions::STRING_OPT,
  "Tests to see what an string value looks like.",
  "stringOptionDefault", NULL
};

static PgOptions::PgOpt BOOL_OPT = {
  "b", "bool-opt", PgOptions::BOOL_OPT,
  "Tests to see what an bool value looks like.",
  "1", NULL
};

 
static PgOptions::PgOpt *optionsDef[] = {
  &BOOL_OPT,
  &INTEGER_OPT,
  &FLOAT_OPT,
  &STRING_OPT,
  &STRING_OPT2,
  NULL
};

#include <string>
using namespace std;

/**
 * @class PgOptionsTest
 * @brief Cppunit class for testing PgOptions.
 */
class PgOptionsTest : public CppUnit::TestFixture
{
  CPPUNIT_TEST_SUITE( PgOptionsTest );
  CPPUNIT_TEST( testConstructor );
  CPPUNIT_TEST( testParsing );
  CPPUNIT_TEST_SUITE_END();

public:
  PgOptions *opts;
  /** Setup before each unit test. */
  void setUp();

  /** Test the construction of arguments. */
  void testConstructor();

  /** Test the parsing of arguments. */
  void testParsing();
};

// Registers the fixture into the 'registry'
CPPUNIT_TEST_SUITE_REGISTRATION( PgOptionsTest );

/** 
 * Override setup to make some options for later testing.
 */
void PgOptionsTest::setUp() {
  string usage("This is my program. Here is where I would give you a brief\n"
	       "description of what it does and how it works. In an ideal world this\n"
	       "text would all wrap nicely at 70 characters or so.\n"
	       "\nusage:\n  "
	       "   myProgram arg1 arg2");
  opts = new PgOptions(usage, optionsDef);
}

/** 
 * Make sure that our options were initialized correctly.
 */
void PgOptionsTest::testConstructor() {
  const PgOptions::PgOpt *o = opts->findPgOpt("integer-opt");
  CPPUNIT_ASSERT(strcmp("10", o->defaultVal) == 0);
}

/** 
 * Are the command line args parsed correctly
 */
void PgOptionsTest::testParsing() {
  char *args[] = {"myProg", "-i", "10", "-b", "hello", "world", 
                  "--float-opt", "-10.5", "-s", "somestring"};
  opts->parseOptions(sizeof(args)/sizeof(char *), args);
  CPPUNIT_ASSERT( opts->boolOpt("b") == true );
  CPPUNIT_ASSERT( opts->intOpt("integer-opt") == 10 );
  CPPUNIT_ASSERT( Convert::doubleCloseEnough(
                                            opts->floatOpt("f"), -10.5));
  CPPUNIT_ASSERT( strcmp(opts->strOpt("string-opt"), "somestring") == 0 );
  CPPUNIT_ASSERT( strcmp(opts->strOpt("s2"), "stringOptionDefault") == 0 );
  CPPUNIT_ASSERT( opts->getNumArgs() == 2 );
  CPPUNIT_ASSERT( strcmp(opts->getArg(0), "hello") == 0 );
  CPPUNIT_ASSERT( strcmp(opts->getArg(1), "world") == 0 );
  
  //  opts->usage();
}


#endif

