////////////////////////////////////////////////////////////////
//
// Copyright (C) 2005 Affymetrix, Inc.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License (version 2) as
// published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program;if not, write to the
//
// Free Software Foundation, Inc.,
// 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
//
////////////////////////////////////////////////////////////////

/* Simple program to check that two matrices are the
   same. */
#include <vector>
#include <string>
#include <map>
#include "util/Convert.h"
#include "util/Err.h"
#include "util/MixedFileCheck.h"
#include "util/PgOptions.h"
#include "util/RowFile.h"
#include "util/TableFile.h"
#include "util/TextFileCheck.h"
#include "util/Verbose.h"

using namespace std;

/**
\page apt-matrix-diff MANUAL: apt-matrix-diff (apt-1.8.0)

\p apt-matrix-diff will check to see if two matrices contain the same values.
This can be annoying to do with a regular diff as differences in
precision and outputting functions can lead to representational
differences for two numbers that are the same (i.e. 1.0 == 1).

\section QuickStart Quick Start
To compare tab-separated files file1.tab and file2.tab, use:

\code
apt-matrix-diff -col-skip 1 -line-skip 1 file1.tab file2.tab

\endcode

\subsection manualOptions Options:

\verbatim
apt-matrix-diff - check to see if two matrices contain the same values.

This can be annoying to do with a regular diff as differences in 
precision and outputting functions can lead to representational 
differences for two numbers that are the same (i.e. 1.0 == 1).

usage:
   apt-matrix-diff -col-skip 1 -line-skip 1 file1.tab file2.tab

options:
   -c, --col-skip       Number of columns (row names?) to skip. 
   -l, --line-skip      Number of lines (header?) to skip. 
   -e, --epsilon        Epsilon tolerance to differences. [default =
                        .001] 
   -p, --print-mismatch Print out entries that don't match. 
   -t, --text           Compare files as straight text, without
                        conversion to numeric data. This behaves in a
                        fashion very similar to diff, except that
                        line-skip number of header lines and all line
                        ending differences are ignored. 
       --mixed          Compare files as a mixture of numeric and
                        non-numeric data. If a field can be converted
                        to a floating point number, compare
                        numerically; if not, compare as text. 
   -m, --match-rows     Try to match rows based on row names (useful
                        for matrices in different orders). 
   -h, --help           This message. 
       --version        Display version information. 
\endverbatim

*/

static const string usage =
"apt-matrix-diff - check to see if two matrices contain the same values.\n\n"
"This can be annoying to do with a regular diff as differences in \n"
"precision and outputting functions can lead to representational \n"
"differences for two numbers that are the same (i.e. 1.0 == 1).\n\n"
"usage:\n"
"   apt-matrix-diff -col-skip 1 -line-skip 1 file1.tab file2.tab";

static PgOptions::PgOpt HELP_OPT = {
  "h", "help", PgOptions::BOOL_OPT,
  "This message.",
  "false", NULL
};

static PgOptions::PgOpt MATCH_OPT = {
  "m", "match-rows", PgOptions::BOOL_OPT,
  "Try to match rows based on row names (useful for matrices in different orders).",
  "false", NULL
};

static PgOptions::PgOpt PRINTMISMATCH_OPT = {
  "p", "print-mismatch", PgOptions::BOOL_OPT,
  "Print out entries that don't match.",
  "false", NULL
};

static PgOptions::PgOpt TEXT_OPT = {
  "t", "text", PgOptions::BOOL_OPT,
  "Compare files as straight text, without conversion to numeric "
  "data.  This behaves in a fashion very similar to diff, except that "
  "line-skip number of header lines and all line ending differences "
  "are ignored.",
  "false", NULL
};

static PgOptions::PgOpt MIXED_OPT = {
  "", "mixed", PgOptions::BOOL_OPT,
  "Compare files as a mixture of numeric and non-numeric data. If a "
  "field can be converted to a floating point number, compare numerically; "
  "if not, compare as text.",
  "false", NULL
};

static PgOptions::PgOpt SKIPLINE_OPT = {
  "l", "line-skip", PgOptions::INT_OPT,
  "Number of lines (header?) to skip.",
  "0", NULL};

static PgOptions::PgOpt SKIPCOL_OPT = {
  "c", "col-skip", PgOptions::INT_OPT,
  "Number of columns (row names?) to skip.",
  "0", NULL};

static PgOptions::PgOpt EPSILON_OPT = {
  "e", "epsilon", PgOptions::FLOAT_OPT,
  "Epsilon tolerance to differences. [default = .001]",
  ".001", NULL};

static PgOptions::PgOpt VERSION_OPT = {
   "", "version", PgOptions::BOOL_OPT,
   "Display version information.",
   "false", NULL};

static PgOptions::PgOpt *optionsDef[] = {
  &SKIPCOL_OPT,
  &SKIPLINE_OPT,
  &EPSILON_OPT,
  &PRINTMISMATCH_OPT,
  &TEXT_OPT,
  &MIXED_OPT,
  &MATCH_OPT,
  &HELP_OPT,
  &VERSION_OPT,
  NULL
};

/**
 * Check each entry in two matrices to see if they are the same.  If
 * doing 'match-rows' we will attempt to find the matching row by the
 * row name.
 *
 * @param opts - Options supplied to program.
 * @return number of entries where difference is greater than epsilon
 */
int matrixDiff(PgOptions *opts) {
  int colSkip = opts->intOpt("col-skip");
  int rowSkip = opts->intOpt("line-skip");
  float epsilon = opts->floatOpt("epsilon");
  bool verbose = opts->boolOpt("print-mismatch");
  const bool text = opts->boolOpt("text");
  const bool mixed = opts->boolOpt("mixed");
  bool matchRows = opts->boolOpt("match-rows");
  if(opts->getNumArgs() != 2) {
    Err::errAbort("Must have exactly two file name arguments.");
  }
  Verbose::setLevel(2);
  if (text) {
    TextFileCheck textCheck (opts->getArg(0), opts->getArg(1), rowSkip);
    string errorMsg;
    const bool textOk = textCheck.check (errorMsg);
    if (! textOk && verbose)
      Verbose::out (1, errorMsg);
    // A return of 0 is good, 1 is bad.
    return ! textOk;
  }
  else if (mixed) {
    MixedFileCheck mixedCheck (opts->getArg(0), opts->getArg(1), (double)epsilon, rowSkip, 0);
    string errorMsg;
    const bool mixedOk = mixedCheck.check (errorMsg);
    if (! mixedOk && verbose)
      Verbose::out (1, errorMsg);
    return ! mixedOk;
  }
  else {
    unsigned int diffCount = Util::matrixDifferences(opts->getArg(0), opts->getArg(1),
                                     colSkip, rowSkip, epsilon, verbose,
                                     matchRows);
    return diffCount;
  }
}

/** Everybody's favorite function. */
int main(int argc, const char *argv[]) {
  const string version ("apt-1.8.0");
  const string cvsId ("$Id: apt-matrix-diff.cpp,v 1.6 2007/05/23 17:55:46 awilli Exp $");

  PgOptions *opts = NULL;
  int same = 0;
  opts = new PgOptions(usage, optionsDef);
  opts->parseOptions(argc, argv);
  // Do we need help?
  if(opts->boolOpt("help") || argc == 1) {
    opts->usage();
    cout << "version:" << endl;
    cout << "   " << version << endl;
    cout << "   " << cvsId << endl;
  }
  else if(opts->boolOpt("version"))
    cout << "version: " << version << " " << cvsId << endl;
  else
    same = matrixDiff(opts);
  delete opts;
  return same;
}
